/**************************************************************************

    HeartBeat - A simple Heart Beat checker for the Palm Computing Platform
    Copyright (C) 2000 Sam Trenholme

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

**************************************************************************/

#include <Pilot.h>				
#include "RachRsc.h"
#ifdef __GNUC__
#include "Callbacks.h"
#endif		

// for definition of IsDigit
#include <CharAttr.h>
#include <SysEvtMgr.h>

// Some defines for displaying the countdown
#define RACHEL_COUNTDOWN_X 50
#define RACHEL_COUNTDOWN_Y 30
#define RACHEL_COUNTDOWN_WIDTH 20
#define RACHEL_COUNTDOWN_HEIGHT 10
#define RACHEL_DIGIT_WIDTH 7

// Global variables
// Variables that start with 'RachPref' are preferences stored between
// sessions.  'RachGlobal' variables are global variables not saved
// between sessions.

int RachGlobalCountdown; // Number of seconds left in countdown

static Err RachelStartApplication(void)
{

	// Open up the main form
        FrmGotoForm(RachelForm);

	// Set the countdown to "off"
	RachGlobalCountdown = -1;

	return 0;
}

static void RachelStopApplication(void)
{

	// All we need to do is close the forms
	FrmCloseAllForms(); 

}

// When they ask for help, give them a help screen
void RachelDoHelp()
{
	FrmAlert(RachelHelpAlert);

}

// Clear out a rectanguar box
// Input: x: left hand side of box
//        y: top of box
//        width:  width of box
//        height: height of box
void RachelClearBox(int x, int y, int width, int height)
{
        RectanglePtr erase_box;

        // They had better keep RectangleType the structure RectanglrPtr
        // points to
        if((erase_box = MemPtrNew(sizeof(RectangleType))) == NULL)
                {
                // Is there a better way to handle memory starvation?
                FrmAlert(RachelOutOfMemory);
                return;
                }

        // Initialize a rectangle the size of the text field we need to erase
        RctSetRectangle(erase_box,x,y,width,height);

        // And erase the text field     
        WinEraseRectangle(erase_box,0);

        // Now, free the memory we allocated above
        MemPtrFree(erase_box);

}

// Print a given number of seconds left (without clearing things)
// Input: seconds: number of seconds left
void RachelShowSeconds(int seconds)
{

	char digits[3]; // String to make the numbers digits

	// Do nothing if they try a number of seconds we can not handle
	if(seconds > 19 || seconds < 0)
		return;

	// And do an itoa-type conversion
	if(seconds < 10)
		digits[0] = '0';
	else
		digits[0] = '1';
	digits[1] = '0' + seconds % 10;
	digits[2] = '\0';

	// Put up the actual number of seconds left
	WinSetUnderlineMode(noUnderline);
	FntSetFont(boldFont);
	WinDrawChars(digits,2,RACHEL_COUNTDOWN_X,RACHEL_COUNTDOWN_Y);

}

// Play some beeps for them
// Input: frequency: frequency of beep in HZ
//        duration: duration of sound in ms 
void RachelPlaySound(int frequency, int duration)
{
        SndCommandType snd;

        snd.cmd = sndCmdFreqDurationAmp;
        snd.param1 = frequency; // frequency
        snd.param2 = duration; // duration (in ms)
        snd.param3 = sndMaxAmp; // volume
        SndDoCmd(0, &snd, 0);

}

// Callback to handle all the events on Rachel's single form
static Boolean RachelHandleEvent(EventPtr event)
{
	Boolean		handled;

#ifdef __GNUC__
	CALLBACK_PROLOGUE
#endif
	handled = false;

	switch (event->eType)
		{

   		case ctlSelectEvent:  // Button was pressed and released.
			// If the timer is not going, start it
			if(RachGlobalCountdown < 0)
				{
				RachGlobalCountdown = 15;
				RachelPlaySound(440,300);
				}
			handled = true;
			break;

		// This is run at startup time
  		case frmOpenEvent:	
			// Physically draw the form
			FrmDrawForm(FrmGetActiveForm());
			handled = true;
			break;
		
		// Has the user selected something on the menu on top?	
		case menuEvent:		
			if (event->data.menu.itemID == RachelMenuAbout)
   				FrmAlert(RachelAboutAlert);
			else
				RachelDoHelp();
			handled = true;
			break;

		}
#ifdef __GNUC__
	CALLBACK_EPILOGUE
#endif
	return(handled);
}


static Boolean RachelHandleEventWrapper(EventPtr event)
{
	FormPtr	frm;
	Int		formId;
	Boolean	handled = false;

	if (event->eType == frmLoadEvent)
		{
		// Load the form resource specified in the event then activate the form.
		formId = event->data.frmLoad.formID;
		frm = FrmInitForm(formId);
		FrmSetActiveForm(frm);

		// Set the event handler for the form.  The handler of the currently 
		// active form is called by FrmDispatchEvent each time it receives an event.
		switch (formId)
			{
			case RachelForm:
				FrmSetEventHandler(frm, RachelHandleEvent);
				break;
			}
		handled = true;
		}
	
	return handled;
}

// Wait one second (minus a little smigin for the time it took to redraw
// the seconds remiaining), processing events as needed
// Input: pointer to event handler (so we can process events here
void RachelWaitOneSecond(EventType *event)
{

	int		ticksToWait;
	Word 		error;  // Needed for event handling
	ULong 		ticksCheck; // Remaining ticks
				    // We track this so events do not
				    // prematurely decrement the timer

	// We want to wait one second
	ticksToWait = SysTicksPerSecond();
	// Take away a few ticks to comensate for the time
	// needed to redraw the timer
	if(ticksToWait > 1000)
		ticksToWait -= 30;
	// Wait one second, grabbing and processing events as needed
	while(ticksToWait > 0)
		{
		ticksCheck = TimGetTicks();
		EvtGetEvent(event, ticksToWait);	
		// Due to the change in the pointer indirection, this looks
		// a little different than the code in the main event loop
                if (! SysHandleEvent(event))
                        if (! MenuHandleEvent(0, event, &error))
                                if (! RachelHandleEventWrapper(event))
                                        FrmDispatchEvent(event);
		// We handle the case of getting an appStopEvent by 
		// leaving this routine (I presume the caller of this routine
		// will check for this particular case)
		if(event->eType == appStopEvent)
			return;
		ticksToWait -= TimGetTicks() - ticksCheck;
		}	

}

static void RachelEventLoop(void)
{
	EventType	event;
	Word		error;
	
	do
		{
		if(RachGlobalCountdown < 0)
			{
			EvtGetEvent(&event, evtWaitForever);
			if (! SysHandleEvent(&event))
				if (! MenuHandleEvent(0, &event, &error))
					if (! RachelHandleEventWrapper(&event))
						FrmDispatchEvent(&event);
			}
		else
			{
			// Draw the seconds left, decrement the timer
			RachelClearBox(RACHEL_COUNTDOWN_X,RACHEL_COUNTDOWN_Y,
				       RACHEL_COUNTDOWN_WIDTH,
				       RACHEL_COUNTDOWN_HEIGHT);
			RachelShowSeconds(RachGlobalCountdown);
			RachGlobalCountdown--;
			// Beep and reset timer if we are out of time
			if(RachGlobalCountdown < 0)
				{
				RachelPlaySound(220,500);
				// The following isn't needed, but I am
				// keeping it so the code stays readable
				RachGlobalCountdown = -1; 
				}
			// The following routine gets and processes events
			RachelWaitOneSecond(&event);
			}
		}
	while (event.eType != appStopEvent);
}


DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Err err = 0;		
	
	if (cmd == sysAppLaunchCmdNormalLaunch)
		{
		if ((err = RachelStartApplication()) == 0)
		{
			RachelEventLoop();
			
			RachelStopApplication();
			}
		}

	// Close all forms to stop memory leaks

	return err;
}
